/**************************************************************************************************
* File: Thumb Wheel click_input
* File Type: C - Source Code File
* Company: (c) mikroElektronika, 2016
* Revision History:
*     - mikroC PRO(AM);
* Description:
*     Each DS2408 IC has a factory-lasered 64-bit registration number.
*     Combined with the standard Dallas Semiconductor 1-Wire protocol it uses to communicate with the target board MCU,
*     the registration number enables multiple devices to reside on the same data line, each one with a unique ID.
*
* Test configuration:
*   Device:          STM32F107VC
*                    http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/CD00220364.pdf
*   Dev.Board:       EasyMx v7 for STM32(R) ARM(R)
*                    http://www.mikroe.com/easymx-pro/stm32/
*                    
*   Oscillator:      HSE-PLL, 72.000MHz
*   Ext. Modules:    None.
*   SW:              mikroC PRO for ARM
*                    http://www.mikroe.com/mikroc/arm/
* NOTES:
*       - Place Thumb Wheel click to the mikroBUS1 socket
*       - Turn on LED's SW15.6 and SW15.7
*
**************************************************************************************************/

/* Constants
 *******************************************************************************/

// Register Addresses
#define DS2408_PIO_LOGIC_REG       0x0088
#define DS2408_PIO_OUTPUT_REG      0x0089
#define DS2408_PIO_ACTIVITY_REG    0x008A
#define DS2408_SEARCH_MASK_REG     0x008B
#define DS2408_SEARCH_SELECT_REG   0x008C
#define DS2408_CONTROL_STATUS_REG  0x008D

// Commands
#define DS2408_PIO_READ_CMD        0xF0
#define DS2408_CHANNEL_READ_CMD    0xF5
#define DS2408_CHANNEL_WRITE_CMD   0x5A
#define DS2408_RESET_CMD           0xC3
#define DS2408_SKIP_ROM_CMD        0xCC

/* Pins
 ******************************************************************************/
sbit DS2408_RST at GPIOC_ODR.B2;

int tmp;

/* Set Configuration Registers
 ******************************************************************************/
void DS2408_SetConfig( char cfg ) 
{
    while( Ow_Reset( &GPIOA_BASE, 0 ) );
  
    Ow_Write( &GPIOA_BASE, 0, DS2408_SKIP_ROM_CMD );                            // One-Wire write data to RC0 pin
    Ow_Write( &GPIOA_BASE, 0, DS2408_SKIP_ROM_CMD );
    Ow_Write( &GPIOA_BASE, 0, DS2408_CONTROL_STATUS_REG );
    Ow_Write( &GPIOA_BASE, 0, DS2408_CONTROL_STATUS_REG >> 8 );
    Ow_Write( &GPIOA_BASE, 0, cfg );
}

/* Software Reset DS2408
 ******************************************************************************/
void DS2408_SoftReset() 
{
    while( Ow_Reset( &GPIOA_BASE, 0 ) );
    
    Ow_Write( &GPIOA_BASE, 0, DS2408_SKIP_ROM_CMD );
    Ow_Write( &GPIOA_BASE, 0, DS2408_RESET_CMD );
}

/* Read DS2408
 ******************************************************************************/
char DS2408_ChannelRead() 
{
    int state;
  
    while( Ow_Reset( &GPIOA_BASE, 0 ) );
    
    Ow_Write( &GPIOA_BASE, 0, DS2408_SKIP_ROM_CMD );
    Ow_Write( &GPIOA_BASE, 0, DS2408_CHANNEL_READ_CMD );
    state = Ow_Read( &GPIOA_BASE, 0 );

    return state;
}

/* Display result on LED's
 ******************************************************************************/
void LED_on( int pos )
{
     GPIOD_ODR = 0;
     
     do
     {
         GPIOD_ODR |= 1 << pos;
     } 
     while( pos-- );
}

/* Main function
 ******************************************************************************/
void main() 
{
    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_ALL );                      // Set PORTD as digital output
    Delay_ms( 100 );

    DS2408_SoftReset();
    Delay_ms( 100 );

    DS2408_SetConfig( 0x01 );
    DS2408_RST = 1;
    Delay_ms( 200 );

    while( 1 )
    {
        tmp = DS2408_ChannelRead();                                             // Read DS2408
        LED_on( tmp );                                                          // Display State on PORTD
    }
}